import com.jscape.inet.mft.management.client.api.ManagerSubsystem;
import com.jscape.inet.mft.management.descriptors.EventServiceDescriptor;
import com.jscape.inet.mft.management.descriptors.ServiceDescriptor;
import com.jscape.inet.mft.transport.events.FileUploadEvent;
import com.jscape.inet.mft.workflow.Trigger;
import com.jscape.inet.mft.workflow.ActionContext;
import com.jscape.inet.mft.workflow.actions.SystemOutAction;
import com.jscape.util.workflow.BinaryExpression;
import com.jscape.util.workflow.ConstantExpression;
import com.jscape.util.workflow.Expression;
import com.jscape.util.workflow.VariableExpression;
import com.jscape.util.workflow.operators.BooleanAndOperator;
import com.jscape.util.workflow.operators.StringEqualsOperator;
import com.jscape.util.workflow.operators.BooleanEqualsOperator;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

public class CreateTriggerExample {

    private static final BufferedReader CONSOLE_READER = new BufferedReader(new InputStreamReader(System.in));

    private static final String DEFAULT_CLIENT_CONFIG_FILE = "../../../client.cfg";

    private ManagerSubsystem manager;
    private String domainName;
    private String configFile;

    /**
     * Collects information about the account.
     *
     * @param args command line arguments
     * @throws Exception if an error occurs
     */
    public static void main(String[] args)
            throws Exception {
        showExampleTitle();
        CreateTriggerExample example = createExample();
        example.execute();
    }

    /**
     * Shows example title.
     */
    private static void showExampleTitle() {
        System.out.println("This example demonstrates how to create a trigger based on File Upload event using the server API.");
        System.out.println();
    }

    /**
     * Creates an example instance.
     *
     * @return an example instance
     * @throws java.io.IOException if an error occurs
     */
    private static CreateTriggerExample createExample()
            throws IOException {
        String domainName = getParameter("Enter domain name: ");
        File configFile = getConfigFile();

        return new CreateTriggerExample(domainName, configFile);
    }

    /**
     * Returns a manager configuration file.
     *
     * @return a manager configuration file
     * @throws java.io.IOException if an I/O error occurs
     */
    private static File getConfigFile()
            throws IOException {
        File configFile = new File(DEFAULT_CLIENT_CONFIG_FILE);

        while (!configFile.exists()) {
            System.out.println("Configuration file not found.");
            String configFilePath = getParameter("Please enter absolute path of client.cfg file: ");
            configFile = new File(configFilePath);
        }

        return configFile;
    }

    /**
     * Support method. Shows prompt on console and reads user response.
     *
     * @param prompt prompt string
     * @return user response
     * @throws java.io.IOException if an I/O error occurs
     */
    private static String getParameter(String prompt)
            throws IOException {
        System.out.print(prompt);
        return CONSOLE_READER.readLine();
    }

    /**
     * Constructs example.
     *
     * @param domainName domain name
     * @param configFile config file
     */
    public CreateTriggerExample(String domainName, File configFile) {
        this.domainName = domainName;
        this.configFile = configFile.getAbsolutePath();
    }

    /**
     * Creates domain and adds to server.
     */
    public void execute() {
        try {
            // create manager
            this.manager = new ManagerSubsystem(this.configFile);

            // establish connection
            this.manager.connect();

            // create service descriptor
            ServiceDescriptor descriptor = createEventServiceDescriptor();

            // set new service descriptor
            this.manager.setEventService(this.domainName, descriptor);

            System.out.println("Event service updated successfully!");
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // disconnect
            this.manager.disconnect();
        }
    }

    /**
     * Creates an event service descriptor.
     *
     * @return a new event service descriptor
     */
    private ServiceDescriptor createEventServiceDescriptor() {
        Trigger trigger = new Trigger();

        // set trigger name
        trigger.setName("Test trigger");

        // set trigger description
        trigger.setDescription("A trigger example.");

        // set trigger event class
        trigger.setEventClass(FileUploadEvent.class);

        // set trigger expression
        Expression expression = createExpression();
        trigger.setExpression(expression);

        // set trigger action
        ActionContext actionContext = createActionContext();
        trigger.setContexts(new ActionContext[] {actionContext});

        return new EventServiceDescriptor(new Trigger[] {trigger});
    }

    /**
     * Creates a simple trigger expression to detect successful "test.txt" file uploading,
     * i.e. "((success EQUALS TRUE) AND (Name EQUALS test.txt))".
     *
     * @return a trigger expression
     */
    private Expression createExpression() {
        // event "success" property verification
        Expression successExpression = new BinaryExpression(new VariableExpression("Success"), new BooleanEqualsOperator(), new ConstantExpression(Boolean.TRUE));

        // event "name" property verification
        Expression filenameExpression = new BinaryExpression(new VariableExpression("Name"), new StringEqualsOperator(), new ConstantExpression("test.txt"));

        return new BinaryExpression(successExpression, new BooleanAndOperator(), filenameExpression);
    }

    /**
     * Creates an action context for SystemOut Action.
     *
     * @return an action context
     */
    private ActionContext createActionContext() {
        ActionContext context = new ActionContext();

        // target action class
        context.setActionClass(SystemOutAction.class);

        // set action property
        context.setParameter("Message", "File %Name% is successfully uploaded.");

        return context;
    }

}
